package live.exit.musicplayer

import android.content.Context
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import android.support.v4.media.session.MediaSessionCompat
import android.support.v4.media.session.PlaybackStateCompat
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import java.io.File
import android.content.BroadcastReceiver
import android.content.Intent
import android.content.IntentFilter
import android.util.Log

const val NOTIFY_PAUSE = "live.exit.musicplayer.pause"
const val NOTIFY_PREVIOUS = "live.exit.musicplayer.previous"
const val NOTIFY_NEXT = "live.exit.musicplayer.next"
const val NOTIFY_PLAY = "live.exit.musicplayer.play"


class MusicPlayerPlugin(private val player: MusicPlayer, private val context: Context, private val session: MediaSessionCompat, private val notificationHandler: NotificationHandler) : BroadcastReceiver(), MethodCallHandler {

  init {
    val filter = IntentFilter()
    filter.addAction(NOTIFY_PAUSE)
    filter.addAction(NOTIFY_PLAY)
    filter.addAction(NOTIFY_PREVIOUS)
    filter.addAction(NOTIFY_NEXT)
    context.registerReceiver(this, filter)
  }

  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val context = registrar.activeContext()
      val channel = MethodChannel(registrar.messenger(), "exit.live/music_player")
      val session = MediaSessionCompat(context, "MediaSession")
      val notificationHandler = NotificationHandler(context, session)
      val player = MusicPlayer(channel, notificationHandler, session)
      val plugin = MusicPlayerPlugin(player, context, session, notificationHandler)
      channel.setMethodCallHandler(plugin)
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
      "play" -> play(call.arguments as Map<String, String>)
      "pause" -> pause()
      "resume" -> resume()
      "seek" -> player.seek(call.arguments as Double)
      "showCover" -> showCover(call.arguments as String)
      else -> {
        result.notImplemented()
      }
    }
    result.success(0)
  }

  private fun play(arguments: Map<String, Any>) {
    session.isActive = true
    session.setPlaybackState(PlaybackStateCompat.Builder()
        .setState(PlaybackStateCompat.STATE_PLAYING, 0, 1f)
        .build())
    val trackName = arguments["trackName"] as String
    val albumName = arguments["albumName"] as String
    val artistName = arguments["artistName"] as String
    notificationHandler.updateNotification(trackName, albumName, artistName)
    val url = arguments["url"] as String
    player.play(url)
  }

  private fun showCover(filename: String) {
    Log.i("showCover", "Got show cover with $filename", null)
    val options = BitmapFactory.Options()
    options.inPreferredConfig = Bitmap.Config.ARGB_8888
    val image = BitmapFactory.decodeFile(File(context.cacheDir, filename).absolutePath, options)
    Log.i("showCover", "Calling updateNotification", null)
    notificationHandler.updateNotification(image)
  }

  private fun pause() {
    notificationHandler.updateNotification(false)
    player.pause()
  }

  private fun resume() {
    notificationHandler.updateNotification(true)
    player.resume()
  }

  override fun onReceive(context: Context?, intent: Intent?) {
    Log.v("MusicPlayerPlugin", "onReceive")
    when {
      intent?.action.equals(NOTIFY_PAUSE) -> {
        Log.v("MusicPlayerPlugin", "notify pause")
        notificationHandler.updateNotification(false)
        player.pause()
      }
      intent?.action.equals(NOTIFY_PLAY) -> {
        Log.v("MusicPlayerPlugin", "notify play")
        notificationHandler.updateNotification(true)
        player.resume()

      }
      intent?.action.equals(NOTIFY_PREVIOUS) -> {
        Log.v("MusicPlayerPlugin", "notify previous")
        player.previous()
      }
      intent?.action.equals(NOTIFY_NEXT) -> {
        Log.v("MusicPlayerPlugin", "notify next")
        notificationHandler.updateNotification(true)
        player.next()
      }
    }
  }
}